@@ -5,12 +5,12 @@
 
     Just export lexer classes previously contained in this module.
 
-    :copyright: Copyright 2006-2014 by the Pygments team, see AUTHORS.
+    :copyright: Copyright 2006-2015 by the Pygments team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from pygments.lexers.lisp import SchemeLexer, CommonLispLexer, RacketLexer, \
-    NewLispLexer
+    NewLispLexer, ShenLexer
 from pygments.lexers.haskell import HaskellLexer, LiterateHaskellLexer, \
     KokaLexer
 from pygments.lexers.theorem import CoqLexer
