@@ -3,7 +3,7 @@
     Pygments regex lexer tests
     ~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: Copyright 2006-2013 by the Pygments team, see AUTHORS.
+    :copyright: Copyright 2006-2015 by the Pygments team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -12,6 +12,7 @@ import unittest
 from pygments.token import Text
 from pygments.lexer import RegexLexer
 from pygments.lexer import bygroups
+from pygments.lexer import default
 
 
 class TestLexer(RegexLexer):
@@ -20,6 +21,7 @@ class TestLexer(RegexLexer):
         'root': [
             ('a', Text.Root, 'rag'),
             ('e', Text.Root),
+            default(('beer', 'beer'))
         ],
         'beer': [
             ('d', Text.Beer, ('#pop', '#pop')),
@@ -45,3 +47,8 @@ class TupleTransTest(unittest.TestCase):
         self.assertEqual(toks,
            [(0, Text.Root, 'a'), (1, Text, u'\n'),
             (2, Text.Root, 'e')])
+
+    def test_default(self):
+        lx = TestLexer()
+        toks = list(lx.get_tokens_unprocessed('d'))
+        self.assertEqual(toks, [(0, Text.Beer, 'd')])
